@@ -1,6 +1,6 @@
 // class template regex -*- C++ -*-
 
-// Copyright (C) 2010-2014 Free Software Foundation, Inc.
+// Copyright (C) 2010-2018 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -31,18 +31,17 @@
 namespace std _GLIBCXX_VISIBILITY(default)
 {
 _GLIBCXX_BEGIN_NAMESPACE_VERSION
+_GLIBCXX_BEGIN_NAMESPACE_CXX11
   template<typename, typename>
     class basic_regex;
 
   template<typename, typename>
     class match_results;
 
-_GLIBCXX_END_NAMESPACE_VERSION
+_GLIBCXX_END_NAMESPACE_CXX11
 
 namespace __detail
 {
-_GLIBCXX_BEGIN_NAMESPACE_VERSION
-
   enum class _RegexExecutorPolicy : int
     { _S_auto, _S_alternate };
 
@@ -51,26 +50,17 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	   _RegexExecutorPolicy __policy,
 	   bool __match_mode>
     bool
-    __regex_algo_impl(_BiIter                              __s,
-		      _BiIter                              __e,
+    __regex_algo_impl(_BiIter			      __s,
+		      _BiIter			      __e,
 		      match_results<_BiIter, _Alloc>&      __m,
 		      const basic_regex<_CharT, _TraitsT>& __re,
 		      regex_constants::match_flag_type     __flags);
 
   template<typename, typename, typename, bool>
     class _Executor;
-
-  template<typename _TraitsT>
-    inline std::shared_ptr<_NFA<_TraitsT>>
-    __compile_nfa(const typename _TraitsT::char_type* __first,
-		  const typename _TraitsT::char_type* __last,
-		  const _TraitsT& __traits,
-		  regex_constants::syntax_option_type __flags);
-
-_GLIBCXX_END_NAMESPACE_VERSION
 }
 
-_GLIBCXX_BEGIN_NAMESPACE_VERSION
+_GLIBCXX_BEGIN_NAMESPACE_CXX11
 
   /**
    * @addtogroup regex
@@ -91,20 +81,17 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     struct regex_traits
     {
     public:
-      typedef _Ch_type                     	char_type;
-      typedef std::basic_string<char_type> 	string_type;
-      typedef std::locale                  	locale_type;
+      typedef _Ch_type				char_type;
+      typedef std::basic_string<char_type>	string_type;
+      typedef std::locale			locale_type;
     private:
       struct _RegexMask
 	{
-	  typedef typename std::ctype<char_type>::mask _BaseType;
+	  typedef std::ctype_base::mask _BaseType;
 	  _BaseType _M_base;
 	  unsigned char _M_extended;
 	  static constexpr unsigned char _S_under = 1 << 0;
-	  // FIXME: _S_blank should be removed in the future,
-	  // when locale's complete.
-	  static constexpr unsigned char _S_blank = 1 << 1;
-	  static constexpr unsigned char _S_valid_mask = 0x3;
+	  static constexpr unsigned char _S_valid_mask = 0x1;
 
 	  constexpr _RegexMask(_BaseType __base = 0,
 			       unsigned char __extended = 0)
@@ -404,8 +391,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 		    "regex traits class must have the same char_type");
 
       // types:
-      typedef _Ch_type                            value_type;
-      typedef _Rx_traits                          traits_type;
+      typedef _Ch_type				  value_type;
+      typedef _Rx_traits			  traits_type;
       typedef typename traits_type::string_type   string_type;
       typedef regex_constants::syntax_option_type flag_type;
       typedef typename traits_type::locale_type   locale_type;
@@ -433,7 +420,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        * character sequence.
        */
       basic_regex()
-      : _M_flags(ECMAScript), _M_automaton(nullptr)
+      : _M_flags(ECMAScript), _M_loc(), _M_automaton(nullptr)
       { }
 
       /**
@@ -474,29 +461,14 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        *
        * @param __rhs A @p regex object.
        */
-      basic_regex(const basic_regex& __rhs)
-      : _M_flags(__rhs._M_flags), _M_original_str(__rhs._M_original_str)
-      {
-	_M_traits.imbue(__rhs.getloc());
-	this->assign(_M_original_str, _M_flags);
-      }
+      basic_regex(const basic_regex& __rhs) = default;
 
       /**
        * @brief Move-constructs a basic regular expression.
        *
        * @param __rhs A @p regex object.
-       *
-       * The implementation is a workaround concerning ABI compatibility. See:
-       * https://gcc.gnu.org/ml/libstdc++/2014-09/msg00067.html
        */
-      basic_regex(basic_regex&& __rhs)
-      : _M_flags(__rhs._M_flags),
-      _M_original_str(std::move(__rhs._M_original_str))
-      {
-	_M_traits.imbue(__rhs.getloc());
-	this->assign(_M_original_str, _M_flags);
-	__rhs._M_automaton.reset();
-      }
+      basic_regex(basic_regex&& __rhs) noexcept = default;
 
       /**
        * @brief Constructs a basic regular expression from the string
@@ -512,7 +484,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	basic_regex(const std::basic_string<_Ch_type, _Ch_traits,
 					    _Ch_alloc>& __s,
 		    flag_type __f = ECMAScript)
-	: basic_regex(__s.begin(), __s.end(), __f)
+	: basic_regex(__s.data(), __s.data() + __s.size(), __f)
 	{ }
 
       /**
@@ -531,13 +503,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       template<typename _FwdIter>
 	basic_regex(_FwdIter __first, _FwdIter __last,
 		    flag_type __f = ECMAScript)
-	: _M_flags(__f),
-	  _M_original_str(__first, __last),
-	  _M_automaton(__detail::__compile_nfa(_M_original_str.c_str(),
-					       _M_original_str.c_str()
-						 + _M_original_str.size(),
-					       _M_traits,
-					       _M_flags))
+	: basic_regex(std::move(__first), std::move(__last), locale_type(), __f)
 	{ }
 
       /**
@@ -567,12 +533,9 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 
       /**
        * @brief Move-assigns one regular expression to another.
-       *
-       * The implementation is a workaround concerning ABI compatibility. See:
-       * https://gcc.gnu.org/ml/libstdc++/2014-09/msg00067.html
        */
       basic_regex&
-      operator=(basic_regex&& __rhs)
+      operator=(basic_regex&& __rhs) noexcept
       { return this->assign(std::move(__rhs)); }
 
       /**
@@ -604,9 +567,9 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        *
        * @param __s A pointer to a string containing a regular expression.
        */
-      template<typename _Ch_typeraits, typename _Alloc>
+      template<typename _Ch_traits, typename _Alloc>
 	basic_regex&
-	operator=(const basic_string<_Ch_type, _Ch_typeraits, _Alloc>& __s)
+	operator=(const basic_string<_Ch_type, _Ch_traits, _Alloc>& __s)
 	{ return this->assign(__s); }
 
       // [7.8.3] assign
@@ -618,10 +581,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       basic_regex&
       assign(const basic_regex& __rhs)
       {
-	_M_flags = __rhs._M_flags;
-	_M_original_str = __rhs._M_original_str;
-	_M_traits.imbue(__rhs.getloc());
-	this->assign(_M_original_str, _M_flags);
+	basic_regex __tmp(__rhs);
+	this->swap(__tmp);
 	return *this;
       }
 
@@ -629,18 +590,12 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        * @brief The move-assignment operator.
        *
        * @param __rhs Another regular expression object.
-       *
-       * The implementation is a workaround concerning ABI compatibility. See:
-       * https://gcc.gnu.org/ml/libstdc++/2014-09/msg00067.html
        */
       basic_regex&
-      assign(basic_regex&& __rhs)
+      assign(basic_regex&& __rhs) noexcept
       {
-	_M_flags = __rhs._M_flags;
-	_M_original_str = std::move(__rhs._M_original_str);
-	__rhs._M_automaton.reset();
-	_M_traits.imbue(__rhs.getloc());
-	this->assign(_M_original_str, _M_flags);
+	basic_regex __tmp(std::move(__rhs));
+	this->swap(__tmp);
 	return *this;
       }
 
@@ -689,16 +644,13 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        * expression pattern interpreted according to @p __flags.  If
        * regex_error is thrown, *this remains unchanged.
        */
-      template<typename _Ch_typeraits, typename _Alloc>
+      template<typename _Ch_traits, typename _Alloc>
 	basic_regex&
-	assign(const basic_string<_Ch_type, _Ch_typeraits, _Alloc>& __s,
+	assign(const basic_string<_Ch_type, _Ch_traits, _Alloc>& __s,
 	       flag_type __flags = ECMAScript)
 	{
-	  _M_automaton = __detail::__compile_nfa(
-	    __s.data(), __s.data() + __s.size(), _M_traits, __flags);
-	  _M_original_str = __s;
-	  _M_flags = __flags;
-	  return *this;
+	  return this->assign(basic_regex(__s.data(), __s.data() + __s.size(),
+					  _M_loc, __flags));
 	}
 
       /**
@@ -765,8 +717,9 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       locale_type
       imbue(locale_type __loc)
       {
+	std::swap(__loc, _M_loc);
 	_M_automaton.reset();
-	return _M_traits.imbue(__loc);
+	return __loc;
       }
 
       /**
@@ -775,7 +728,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        */
       locale_type
       getloc() const
-      { return _M_traits.getloc(); }
+      { return _M_loc; }
 
       // [7.8.6] swap
       /**
@@ -787,10 +740,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       swap(basic_regex& __rhs)
       {
 	std::swap(_M_flags, __rhs._M_flags);
-	std::swap(_M_traits, __rhs._M_traits);
-	auto __tmp = std::move(_M_original_str);
-	this->assign(__rhs._M_original_str, _M_flags);
-	__rhs.assign(__tmp, __rhs._M_flags);
+	std::swap(_M_loc, __rhs._M_loc);
+	std::swap(_M_automaton, __rhs._M_automaton);
       }
 
 #ifdef _GLIBCXX_DEBUG
@@ -800,7 +751,15 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 #endif
 
     private:
-      typedef std::shared_ptr<__detail::_NFA<_Rx_traits>> _AutomatonPtr;
+      typedef std::shared_ptr<const __detail::_NFA<_Rx_traits>> _AutomatonPtr;
+
+      template<typename _FwdIter>
+	basic_regex(_FwdIter __first, _FwdIter __last, locale_type __loc,
+		    flag_type __f)
+	: _M_flags(__f), _M_loc(std::move(__loc)),
+	_M_automaton(__detail::__compile_nfa<_Rx_traits>(
+	  std::move(__first), std::move(__last), _M_loc, _M_flags))
+	{ }
 
       template<typename _Bp, typename _Ap, typename _Cp, typename _Rp,
 	__detail::_RegexExecutorPolicy, bool>
@@ -812,12 +771,60 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       template<typename, typename, typename, bool>
 	friend class __detail::_Executor;
 
-      flag_type              _M_flags;
-      _Rx_traits             _M_traits;
-      basic_string<_Ch_type> _M_original_str;
-      _AutomatonPtr          _M_automaton;
+      flag_type		_M_flags;
+      locale_type	_M_loc;
+      _AutomatonPtr	_M_automaton;
     };
 
+#if __cplusplus < 201703L
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::icase;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::nosubs;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::optimize;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::collate;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::ECMAScript;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::basic;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::extended;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::awk;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::grep;
+
+  template<typename _Ch, typename _Tr>
+    constexpr regex_constants::syntax_option_type
+    basic_regex<_Ch, _Tr>::egrep;
+#endif // ! C++17
+
+#if __cpp_deduction_guides >= 201606
+  template<typename _ForwardIterator>
+    basic_regex(_ForwardIterator, _ForwardIterator,
+		regex_constants::syntax_option_type = {})
+      -> basic_regex<typename iterator_traits<_ForwardIterator>::value_type>;
+#endif
+
   /** @brief Standard regular expressions. */
   typedef basic_regex<char>    regex;
 
@@ -861,8 +868,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     public:
       typedef typename __iter_traits::value_type      	value_type;
       typedef typename __iter_traits::difference_type 	difference_type;
-      typedef _BiIter                                   iterator;
-      typedef std::basic_string<value_type>             string_type;
+      typedef _BiIter				   iterator;
+      typedef std::basic_string<value_type>	     string_type;
 
       bool matched;
 
@@ -947,14 +954,14 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 
 
   /** @brief Standard regex submatch over a C-style null-terminated string. */
-  typedef sub_match<const char*>             csub_match;
+  typedef sub_match<const char*>	     csub_match;
 
   /** @brief Standard regex submatch over a standard string. */
   typedef sub_match<string::const_iterator>  ssub_match;
 
 #ifdef _GLIBCXX_USE_WCHAR_T
   /** @brief Regex submatch over a C-style null-terminated wide string. */
-  typedef sub_match<const wchar_t*>          wcsub_match;
+  typedef sub_match<const wchar_t*>	  wcsub_match;
 
   /** @brief Regex submatch over a standard wide string. */
   typedef sub_match<wstring::const_iterator> wssub_match;
@@ -1045,7 +1052,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     inline bool
     operator==(const __sub_match_string<_Bi_iter, _Ch_traits, _Ch_alloc>& __lhs,
 	       const sub_match<_Bi_iter>& __rhs)
-    { return __rhs.compare(__lhs.c_str()) == 0; }
+    {
+      typedef typename sub_match<_Bi_iter>::string_type string_type;
+      return __rhs.compare(string_type(__lhs.data(), __lhs.size())) == 0;
+    }
 
   /**
    * @brief Tests the inequivalence of a string and a regular expression
@@ -1070,7 +1080,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     inline bool
     operator<(const __sub_match_string<_Bi_iter, _Ch_traits, _Ch_alloc>& __lhs,
 	      const sub_match<_Bi_iter>& __rhs)
-     { return __rhs.compare(__lhs.c_str()) > 0; }
+    {
+      typedef typename sub_match<_Bi_iter>::string_type string_type;
+      return __rhs.compare(string_type(__lhs.data(), __lhs.size())) > 0;
+    }
 
   /**
    * @brief Tests the ordering of a string and a regular expression submatch.
@@ -1119,7 +1132,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     inline bool
     operator==(const sub_match<_Bi_iter>& __lhs,
 	       const __sub_match_string<_Bi_iter, _Ch_traits, _Ch_alloc>& __rhs)
-    { return __lhs.compare(__rhs.c_str()) == 0; }
+    {
+      typedef typename sub_match<_Bi_iter>::string_type string_type;
+      return __lhs.compare(string_type(__rhs.data(), __rhs.size())) == 0;
+    }
 
   /**
    * @brief Tests the inequivalence of a regular expression submatch and a
@@ -1144,7 +1160,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     inline bool
     operator<(const sub_match<_Bi_iter>& __lhs,
 	      const __sub_match_string<_Bi_iter, _Ch_traits, _Ch_alloc>& __rhs)
-    { return __lhs.compare(__rhs.c_str()) < 0; }
+    {
+      typedef typename sub_match<_Bi_iter>::string_type string_type;
+      return __lhs.compare(string_type(__rhs.data(), __rhs.size())) < 0;
+    }
 
   /**
    * @brief Tests the ordering of a regular expression submatch and a string.
@@ -1507,17 +1526,6 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   // [7.10] Class template match_results
 
-  /*
-   * Special sub_match object representing an unmatched sub-expression.
-   */
-  template<typename _Bi_iter>
-    inline const sub_match<_Bi_iter>&
-    __unmatched_sub()
-    {
-      static const sub_match<_Bi_iter> __unmatched = sub_match<_Bi_iter>();
-      return __unmatched;
-    }
-
   /**
    * @brief The results of a match or search operation.
    *
@@ -1547,15 +1555,20 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     {
     private:
       /*
-       * The vector base is empty if this does not represent a successful match.
-       * Otherwise it contains n+3 elements where n is the number of marked
+       * The vector base is empty if this does not represent a match (!ready());
+       * Otherwise if it's a match failure, it contains 3 elements:
+       * [0] unmatched
+       * [1] prefix
+       * [2] suffix
+       * Otherwise it contains n+4 elements where n is the number of marked
        * sub-expressions:
        * [0] entire match
        * [1] 1st marked subexpression
        * ...
        * [n] nth marked subexpression
-       * [n+1] prefix
-       * [n+2] suffix
+       * [n+1] unmatched
+       * [n+2] prefix
+       * [n+3] suffix
        */
       typedef std::vector<sub_match<_Bi_iter>, _Alloc>     _Base_type;
       typedef std::iterator_traits<_Bi_iter>   	   	   __iter_traits;
@@ -1566,16 +1579,16 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        * @name 10.? Public Types
        */
       //@{
-      typedef sub_match<_Bi_iter>                          value_type;
-      typedef const value_type&                            const_reference;
-      typedef const_reference                              reference;
-      typedef typename _Base_type::const_iterator          const_iterator;
-      typedef const_iterator                               iterator;
+      typedef sub_match<_Bi_iter>			   value_type;
+      typedef const value_type&				   const_reference;
+      typedef value_type&				   reference;
+      typedef typename _Base_type::const_iterator	   const_iterator;
+      typedef const_iterator				   iterator;
       typedef typename __iter_traits::difference_type	   difference_type;
       typedef typename allocator_traits<_Alloc>::size_type size_type;
-      typedef _Alloc                                       allocator_type;
+      typedef _Alloc					   allocator_type;
       typedef typename __iter_traits::value_type 	   char_type;
-      typedef std::basic_string<char_type>                 string_type;
+      typedef std::basic_string<char_type>		   string_type;
       //@}
 
     public:
@@ -1647,10 +1660,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        */
       size_type
       size() const
-      {
-      	size_type __size = _Base_type::size();
-      	return (__size && _Base_type::operator[](0).matched) ? __size - 2 : 0;
-      }
+      { return _Base_type::empty() ? 0 : _Base_type::size() - 3; }
 
       size_type
       max_size() const
@@ -1694,15 +1704,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        * is zero (the default), in which case this function returns the offset
        * from the beginning of the target sequence to the beginning of the
        * match.
-       *
-       * Returns -1 if @p __sub is out of range.
        */
       difference_type
       position(size_type __sub = 0) const
-      {
-	return __sub < size() ? std::distance(_M_begin,
-					      (*this)[__sub].first) : -1;
-      }
+      { return std::distance(_M_begin, (*this)[__sub].first); }
 
       /**
        * @brief Gets the match or submatch converted to a string type.
@@ -1715,7 +1720,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        */
       string_type
       str(size_type __sub = 0) const
-      { return (*this)[__sub].str(); }
+      { return string_type((*this)[__sub]); }
 
       /**
        * @brief Gets a %sub_match reference for the match or submatch.
@@ -1731,10 +1736,10 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       const_reference
       operator[](size_type __sub) const
       {
-      	_GLIBCXX_DEBUG_ASSERT( ready() );
-      	return __sub < size()
-	       ?  _Base_type::operator[](__sub)
-	       : __unmatched_sub<_Bi_iter>();
+	__glibcxx_assert( ready() );
+	return __sub < size()
+	       ? _Base_type::operator[](__sub)
+	       : _M_unmatched_sub();
       }
 
       /**
@@ -1748,10 +1753,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       const_reference
       prefix() const
       {
-      	_GLIBCXX_DEBUG_ASSERT( ready() );
-      	return !empty()
-      	       ? _Base_type::operator[](_Base_type::size() - 2)
-	       : __unmatched_sub<_Bi_iter>();
+	__glibcxx_assert( ready() );
+	return !empty() ? _M_prefix() : _M_unmatched_sub();
       }
 
       /**
@@ -1765,10 +1768,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       const_reference
       suffix() const
       {
-	_GLIBCXX_DEBUG_ASSERT( ready() );
-	return !empty()
-	       ? _Base_type::operator[](_Base_type::size() - 1)
-	       : __unmatched_sub<_Bi_iter>();
+	__glibcxx_assert( ready() );
+	return !empty() ? _M_suffix() : _M_unmatched_sub();
       }
 
       /**
@@ -1790,7 +1791,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
        */
       const_iterator
       end() const
-      { return _Base_type::end() - 2; }
+      { return _Base_type::end() - (empty() ? 0 : 3); }
 
       /**
        * @brief Gets an iterator to one-past-the-end of the collection.
@@ -1907,14 +1908,41 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 				    const basic_regex<_Cp, _Rp>&,
 				    regex_constants::match_flag_type);
 
+      void
+      _M_resize(unsigned int __size)
+      { _Base_type::resize(__size + 3); }
+
+      const_reference
+      _M_unmatched_sub() const
+      { return _Base_type::operator[](_Base_type::size() - 3); }
+
+      sub_match<_Bi_iter>&
+      _M_unmatched_sub()
+      { return _Base_type::operator[](_Base_type::size() - 3); }
+
+      const_reference
+      _M_prefix() const
+      { return _Base_type::operator[](_Base_type::size() - 2); }
+
+      sub_match<_Bi_iter>&
+      _M_prefix()
+      { return _Base_type::operator[](_Base_type::size() - 2); }
+
+      const_reference
+      _M_suffix() const
+      { return _Base_type::operator[](_Base_type::size() - 1); }
+
+      sub_match<_Bi_iter>&
+      _M_suffix()
+      { return _Base_type::operator[](_Base_type::size() - 1); }
+
       _Bi_iter _M_begin;
-      bool     _M_in_iterator;
     };
 
-  typedef match_results<const char*>             cmatch;
-  typedef match_results<string::const_iterator>  smatch;
+  typedef match_results<const char*>		 cmatch;
+  typedef match_results<string::const_iterator>	 smatch;
 #ifdef _GLIBCXX_USE_WCHAR_T
-  typedef match_results<const wchar_t*>          wcmatch;
+  typedef match_results<const wchar_t*>		 wcmatch;
   typedef match_results<wstring::const_iterator> wsmatch;
 #endif
 
@@ -1968,6 +1996,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	 match_results<_Bi_iter, _Alloc>& __rhs)
     { __lhs.swap(__rhs); }
 
+_GLIBCXX_END_NAMESPACE_CXX11
+
   // [7.11.2] Function template regex_match
   /**
    * @name Matching, Searching, and Replacing
@@ -1992,11 +2022,11 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
   template<typename _Bi_iter, typename _Alloc,
 	   typename _Ch_type, typename _Rx_traits>
     inline bool
-    regex_match(_Bi_iter                                 __s,
-		_Bi_iter                                 __e,
-		match_results<_Bi_iter, _Alloc>&         __m,
+    regex_match(_Bi_iter				 __s,
+		_Bi_iter				 __e,
+		match_results<_Bi_iter, _Alloc>&	 __m,
 		const basic_regex<_Ch_type, _Rx_traits>& __re,
-		regex_constants::match_flag_type         __flags
+		regex_constants::match_flag_type	 __flags
 			       = regex_constants::match_default)
     {
       return __detail::__regex_algo_impl<_Bi_iter, _Alloc, _Ch_type, _Rx_traits,
@@ -2077,6 +2107,19 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 		= regex_constants::match_default)
     { return regex_match(__s.begin(), __s.end(), __m, __re, __flags); }
 
+  // _GLIBCXX_RESOLVE_LIB_DEFECTS
+  // 2329. regex_match() with match_results should forbid temporary strings
+  /// Prevent unsafe attempts to get match_results from a temporary string.
+  template<typename _Ch_traits, typename _Ch_alloc,
+	   typename _Alloc, typename _Ch_type, typename _Rx_traits>
+    bool
+    regex_match(const basic_string<_Ch_type, _Ch_traits, _Ch_alloc>&&,
+		match_results<typename basic_string<_Ch_type,
+		_Ch_traits, _Ch_alloc>::const_iterator, _Alloc>&,
+		const basic_regex<_Ch_type, _Rx_traits>&,
+		regex_constants::match_flag_type
+		= regex_constants::match_default) = delete;
+
   /**
    * @brief Indicates if there is a match between the regular expression @p e
    * and a C-style null-terminated string.
@@ -2253,6 +2296,20 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 		 = regex_constants::match_default)
     { return regex_search(__s.begin(), __s.end(), __m, __e, __f); }
 
+  // _GLIBCXX_RESOLVE_LIB_DEFECTS
+  // 2329. regex_search() with match_results should forbid temporary strings
+  /// Prevent unsafe attempts to get match_results from a temporary string.
+  template<typename _Ch_traits, typename _Ch_alloc,
+	   typename _Alloc, typename _Ch_type,
+	   typename _Rx_traits>
+    bool
+    regex_search(const basic_string<_Ch_type, _Ch_traits, _Ch_alloc>&&,
+		 match_results<typename basic_string<_Ch_type,
+		 _Ch_traits, _Ch_alloc>::const_iterator, _Alloc>&,
+		 const basic_regex<_Ch_type, _Rx_traits>&,
+		 regex_constants::match_flag_type
+		 = regex_constants::match_default) = delete;
+
   // std [28.11.4] Function template regex_replace
   /**
    * @brief Search for a regular expression within a range for multiple times,
@@ -2409,6 +2466,8 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   //@}
 
+_GLIBCXX_BEGIN_NAMESPACE_CXX11
+
   // std [28.12] Class template regex_iterator
   /**
    * An iterator adaptor that will provide repeated calls of regex_search over
@@ -2421,18 +2480,18 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     {
     public:
       typedef basic_regex<_Ch_type, _Rx_traits>  regex_type;
-      typedef match_results<_Bi_iter>            value_type;
-      typedef std::ptrdiff_t                     difference_type;
-      typedef const value_type*                  pointer;
-      typedef const value_type&                  reference;
-      typedef std::forward_iterator_tag          iterator_category;
+      typedef match_results<_Bi_iter>	    value_type;
+      typedef std::ptrdiff_t		     difference_type;
+      typedef const value_type*		  pointer;
+      typedef const value_type&		  reference;
+      typedef std::forward_iterator_tag	  iterator_category;
 
       /**
        * @brief Provides a singular iterator, useful for indicating
        * one-past-the-end of a range.
        */
       regex_iterator()
-      : _M_match()
+      : _M_pregex()
       { }
 
       /**
@@ -2451,6 +2510,11 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	  *this = regex_iterator();
       }
 
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 2332. regex_iterator should forbid temporary regexes
+      regex_iterator(_Bi_iter, _Bi_iter, const regex_type&&,
+		     regex_constants::match_flag_type
+		     = regex_constants::match_default) = delete;
       /**
        * Copy constructs a %regex_iterator.
        */
@@ -2507,18 +2571,18 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       }
 
     private:
-      _Bi_iter                         _M_begin;
-      _Bi_iter                         _M_end;
-      const regex_type*                _M_pregex;
-      regex_constants::match_flag_type _M_flags;
-      match_results<_Bi_iter>          _M_match;
+      _Bi_iter				_M_begin;
+      _Bi_iter				_M_end;
+      const regex_type*			_M_pregex;
+      regex_constants::match_flag_type	_M_flags;
+      match_results<_Bi_iter>		_M_match;
     };
 
-  typedef regex_iterator<const char*>             cregex_iterator;
-  typedef regex_iterator<string::const_iterator>  sregex_iterator;
+  typedef regex_iterator<const char*>			cregex_iterator;
+  typedef regex_iterator<string::const_iterator>	sregex_iterator;
 #ifdef _GLIBCXX_USE_WCHAR_T
-  typedef regex_iterator<const wchar_t*>          wcregex_iterator;
-  typedef regex_iterator<wstring::const_iterator> wsregex_iterator;
+  typedef regex_iterator<const wchar_t*>		wcregex_iterator;
+  typedef regex_iterator<wstring::const_iterator>	wsregex_iterator;
 #endif
 
   // [7.12.2] Class template regex_token_iterator
@@ -2535,12 +2599,12 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
     class regex_token_iterator
     {
     public:
-      typedef basic_regex<_Ch_type, _Rx_traits> regex_type;
-      typedef sub_match<_Bi_iter>               value_type;
-      typedef std::ptrdiff_t                    difference_type;
-      typedef const value_type*                 pointer;
-      typedef const value_type&                 reference;
-      typedef std::forward_iterator_tag         iterator_category;
+      typedef basic_regex<_Ch_type, _Rx_traits>	regex_type;
+      typedef sub_match<_Bi_iter>		value_type;
+      typedef std::ptrdiff_t			difference_type;
+      typedef const value_type*			pointer;
+      typedef const value_type&			reference;
+      typedef std::forward_iterator_tag		iterator_category;
 
     public:
       /**
@@ -2632,6 +2696,25 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
       _M_subs(__submatches, __submatches + _Nm), _M_n(0)
       { _M_init(__a, __b); }
 
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 2332. regex_token_iterator should forbid temporary regexes
+      regex_token_iterator(_Bi_iter, _Bi_iter, const regex_type&&, int = 0,
+			   regex_constants::match_flag_type =
+			   regex_constants::match_default) = delete;
+      regex_token_iterator(_Bi_iter, _Bi_iter, const regex_type&&,
+			   const std::vector<int>&,
+			   regex_constants::match_flag_type =
+			   regex_constants::match_default) = delete;
+      regex_token_iterator(_Bi_iter, _Bi_iter, const regex_type&&,
+			   initializer_list<int>,
+			   regex_constants::match_flag_type =
+			   regex_constants::match_default) = delete;
+      template <std::size_t _Nm>
+	regex_token_iterator(_Bi_iter, _Bi_iter, const regex_type&&,
+			     const int (&)[_Nm],
+			     regex_constants::match_flag_type =
+			     regex_constants::match_default) = delete;
+
       /**
        * @brief Copy constructs a %regex_token_iterator.
        * @param __rhs [IN] A %regex_token_iterator to copy.
@@ -2723,31 +2806,33 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	  _M_result = nullptr;
       }
 
-      _Position         _M_position;
-      std::vector<int>  _M_subs;
-      value_type        _M_suffix;
-      std::size_t       _M_n;
-      const value_type* _M_result;
+      _Position		_M_position;
+      std::vector<int>	_M_subs;
+      value_type	_M_suffix;
+      std::size_t	_M_n;
+      const value_type*	_M_result;
 
       // Show whether _M_subs contains -1
-      bool              _M_has_m1;
+      bool		_M_has_m1;
     };
 
   /** @brief Token iterator for C-style NULL-terminated strings. */
-  typedef regex_token_iterator<const char*>             cregex_token_iterator;
+  typedef regex_token_iterator<const char*>		cregex_token_iterator;
 
   /** @brief Token iterator for standard strings. */
-  typedef regex_token_iterator<string::const_iterator>  sregex_token_iterator;
+  typedef regex_token_iterator<string::const_iterator>	sregex_token_iterator;
 
 #ifdef _GLIBCXX_USE_WCHAR_T
   /** @brief Token iterator for C-style NULL-terminated wide strings. */
-  typedef regex_token_iterator<const wchar_t*>          wcregex_token_iterator;
+  typedef regex_token_iterator<const wchar_t*>		wcregex_token_iterator;
 
   /** @brief Token iterator for standard wide-character strings. */
   typedef regex_token_iterator<wstring::const_iterator> wsregex_token_iterator;
 #endif
 
   //@} // group regex
+
+_GLIBCXX_END_NAMESPACE_CXX11
 _GLIBCXX_END_NAMESPACE_VERSION
 } // namespace
 
